.\" $RCSfile: reyesxcpt.3,v $ $Revision: #1 $ (Pixar) $Date: 2004/06/16 $
.ds ]W "Pixar \- Reyes
.TH REYESXCPT 3 "$Date: 2004/06/16 $"
.SH NAME
.ta 1.5i
reyesxcpt	\- Reyes exception-handling conventions
.SH SYNOPSIS
.nf
#include <ri.h>
#include <xcpt.h>

Xcpt(char *xid, int code, int severity, ...)
.fi
.SH DESCRIPTION
This manual page describes Reyes-specific software and conventions for
use of the exception handling scheme described in
.IR xcpt (3),
.IR xcpt (5),
.IR xcptlist (1),
.IR xcptmake (1),
and
.IR xcptmsgs (1).
.LP
The basic technique described in
.IR xcpt (3)
and
.IR xcpt (5)
is used, with the error codes and severity levels obtained from the
RenderMan Interface definitions file
.BR ri.h .
The subroutine
.I Xcpt\^
is called in order to handle an
abnormal condition detected in a program.
The message files for Reyes live in the directory
.B /usr/rman/reyes/lib/messages
and have names of the form \fIletter\fB.msg\fR, where the \fIletter\fR
is the subsystem identification letter.
(At a later time, it may be useful to modify the naming convention so that
the file names indicate the language of the messages (English, French, etc.)
as well as the subsystem ID.)
.LP
In Reyes, the
.I XcptHandler\^
routine described in
.IR xcpt (3)
is implemented in the \fCerror.c\fR file of \fBlibrireyes\fR.
.I XcptHandler\^
calls an application specific handler, either one of the built-in ones
(\fCRiErrorIgnore\fR, \fCRiErrorPrint\fR, and \fCRiErrorAbort\fR) or an
application-defined handler passed into \fCRiErrorMode\fR.
All of these handlers have the same parameters as
.IR XcptHandler .
The handler can take an action which is conditional on the error code or
severity, and it has the formatted message available to print or display
any way it chooses.
The Reyes code should \fInever\fR exit or loop when an exception is
encountered.  It is important that the decision to exit is deferred to
the handler.
This means that all calls to
.I Xcpt\^
must be assumed to return.
All of the obsolete ``fatal error'' stuff in Reyes must be rewritten to
recover and continue after calling
.IR Xcpt .
.SH "RI ERROR CODES AND SEVERITY LEVELS
The RI error codes are defined in
.B ri.h
and are passed to the application program which invoked Reyes.
The intention is that the error code should give the application some
indication of the type of error which has occurred and the action which
might be taken.
The philosophy behind error codes is that different error codes should
be assigned to conditions which might reasonably be handled differently
by the application.
Therefore, several different codes which once indicated distinct types
of program bugs have been replaced by a single RIE_BUG code.
More specific information about the type of bug or the location at which
it occurred is contained in the message and (implicitly) in the unique
exception identifier which is attached to each message.
The currently defined error codes are as follows:

.ta 1.5i
.nf
RIE_NOERROR	0 (no error has occurred)

.fi
\fIThe following codes result from program bugs, hardware
or OS bugs,
system resource limits, and problems with files.\fR
.nf
RIE_NOMEM	out of memory (can't malloc)
RIE_SYSTEM	operating system error
RIE_NOFILE	nonexistent file
RIE_BADFILE	bad file format
RIE_VERSION	version mismatch (protocol, file format)
RIE_BUG		probably a bug in Reyes
RIE_BOTCH	obsolete (same as RIE_BUG)
RIE_CORRUPTED	obsolete (same as RIE_BUG)
RIE_ZERODIVIDE	obsolete (same as RIE_BUG)
RIE_BADPERSP	obsolete (same as RIE_BUG)
RIE_NOSHADEOP	obsolete (same as RIE_BUG)
RIE_LOSTDATA	obsolete (same as RIE_BUG)

\fIThe following codes result from program limitations.\fR
RIE_LIMIT	arbitrary program limit exceeded
RIE_TOOBIG	obsolete (same as RIE_LIMIT)
RIE_INCAPABLE	optional RI feature not in Reyes
RIE_UNIMPLEMENT	mandatory RI feature not in Reyes (yet)

\fIThe following errors are probably due to misuse of RI.\fR
RIE_NOTSTARTED	no RiBegin
RIE_NESTING	bad begin-end nesting
RIE_NOTOPTIONS	unknown option
RIE_NOTATTRIBS	unknown attribute
RIE_NOTPRIMS	unknown primitive
RIE_ILLSTATE	RI in illegal state
RIE_BADMOTION	bad motionbegin-end block
RIE_BADSOLID	bad solidbegin-end block
RIE_BADTOKEN	unknown RI protocol token
RIE_RANGE	parameter out of range
RIE_CONSISTENCY	parameters inconsistent
RIE_BADHANDLE	bad handle passed to RI routine
RIE_NOSHADER	nonexistent or bad shader
RIE_MISSINGDATA	missing data in parameters
RIE_SYNTAX	syntax error in RiDeclare
RIE_BADTYPE	unknown patch type
RIE_BADWRAP	unknown wrap mode type (RiPatchMesh)
.fi

Error severity levels are as indicated below.
Remember that RIE_SEVERE is handled in the same way as RIE_ERROR.
The only extra information conveyed by RIE_SEVERE is that the Reyes
programmer felt that the error was especially severe and that the
application would be well advised to abort after such an error.
The application may choose \fInot\fR to abort, so Reyes must be
able to continue operating correctly even after reporting an RIE_SEVERE
error.
If necessary, flags may be raised within Reyes to disable certain
parts of the system after severe errors if it is felt that proper operation
of those parts of the system cannot be guaranteed.
The built-in Reyes error handlers never abort on an exception with
RIE_WARNING severity, and always abort on an exception with RIE_SEVERE
severity, but this behavior should not be relied upon.

.ta 1.5i
.nf
RIE_WARNING	something seems wrong, but we don't know for sure
RIE_ERROR	a definite problem; results will probably be wrong
RIE_SEVERE	a severe problem; recommend that the application stop Reyes
.fi
.SH "SUBSYSTEM DESIGNATIONS
Each ``subsystem'' of Reyes must be assigned a letter to use in giving
unique identifiers to source code files.
It is most convenient if a letter (or at least a specified range of
file identifiers) is assigned to each source code directory.
Here are the identifiers which have been chosen so far:

.ta 1i
.nf
A00-A99	librip
B00-B99	Reyes utility programs (bin)
C00-C69	libshadecom
C70-C99	shader compiler (other)
D00-D99	libdspy
G00-G99	librireyes/primitive
H00-H99	librireyes/hider
P00-P99	librip.parser
R00-R49	librireyes/ri
R50-R89	librireyes/reyes
R90-R99	librireyes/display
S00-S99	librireyes/shadeop
T00-T49	libtext
T50-T99	texture utility programs (bin)
X00-X99	server programs (etc)
.fi
.SH "SEE ALSO
.IR xcpt (3),
.IR xcpt (5),
.IR xcptlist (1),
.IR xcptmake (1),
.IR xcptmsgs (1).
